#ifndef CONDITIONCHECKEQUALSPOSITIONWITHTOLERANCE_H
#define CONDITIONCHECKEQUALSPOSITIONWITHTOLERANCE_H

#include <Core/core/system/ImportExport.h>
#include <Core/observers/ConditionCheck.h>
#include <Core/robotstate/remote/ArmarPose.h>

namespace armarx
{
    class ARMARXCORE_IMPORT_EXPORT ConditionCheckEqualsPoseWithTolerance :
        public ConditionCheck
    {
    public:
        ConditionCheckEqualsPoseWithTolerance()
        {
            setNumberParameters(2);
            addSupportedType(VariantType::FramedPosition, createParameterTypeList(2, VariantType::FramedPosition, VariantType::Float));
            addSupportedType(VariantType::FramedOrientation, createParameterTypeList(2, VariantType::FramedOrientation, VariantType::Float));
            addSupportedType(VariantType::Vector3, createParameterTypeList(2, VariantType::Vector3, VariantType::Float));
            addSupportedType(VariantType::Quaternion, createParameterTypeList(2, VariantType::Quaternion, VariantType::Float));
            addSupportedType(VariantType::Pose, createParameterTypeList(2, VariantType::Pose, VariantType::Float));
        }

        ConditionCheck* clone()
        {
            return new ConditionCheckEqualsPoseWithTolerance(*this);
        }

        bool evaluate(const StringVariantMap &dataFields)
        {
            if(dataFields.size() != 1)
            {
                printf("Size of dataFields: %d\n", (int)dataFields.size());
                throw InvalidConditionException("Wrong number of datafields for condition equals ");
            }

            Variant& value = dataFields.begin()->second;
            VariantTypeId type = value.getType();

            if(type == VariantType::Vector3)
            {
                Vector3Ptr & typedValue =  value.getClass<Vector3>();
                Vector3Ptr & param =  getParameter(0).getClass<Vector3>();
                return (sqrt(((typedValue->x-param->x)*(typedValue->x-param->x))+
                             ((typedValue->y-param->y)*(typedValue->y-param->y))+
                             ((typedValue->z-param->x)*(typedValue->x-param->z))) < getParameter(1).getFloat());
            }

            if(type == VariantType::Quaternion)
            {
                QuaternionPtr & typedValue =  value.getClass<Quaternion>();
                QuaternionPtr & param =  getParameter(0).getClass<Quaternion>();
                return (sqrt(((typedValue->qw-param->qw)*(typedValue->qw-param->qw))+
                             ((typedValue->qx-param->qx)*(typedValue->qx-param->qx))+
                             ((typedValue->qy-param->qy)*(typedValue->qy-param->qy))+
                             ((typedValue->qz-param->qx)*(typedValue->qx-param->qz))) < getParameter(1).getFloat());
            }


            if(type == VariantType::FramedPosition)
            {
                FramedPositionPtr & typedValue =  value.getClass<FramedPosition>();
                FramedPositionPtr & param =  getParameter(0).getClass<FramedPosition>();
                return param->getFrame() == typedValue->getFrame()
                        && (sqrt(((typedValue->x-param->x)*(typedValue->x-param->x))+
                             ((typedValue->y-param->y)*(typedValue->y-param->y))+
                             ((typedValue->z-param->x)*(typedValue->x-param->z))) < getParameter(1).getFloat());
            }


            if(type == VariantType::FramedOrientation)
            {
                FramedOrientationPtr & typedValue =  value.getClass<FramedOrientation>();
                FramedOrientationPtr & param =  getParameter(0).getClass<FramedOrientation>();
                return param->getFrame() == typedValue->getFrame()
                        &&(sqrt(((typedValue->qw-param->qw)*(typedValue->qw-param->qw))+
                             ((typedValue->qx-param->qx)*(typedValue->qx-param->qx))+
                             ((typedValue->qy-param->qy)*(typedValue->qy-param->qy))+
                             ((typedValue->qz-param->qx)*(typedValue->qx-param->qz))) < getParameter(1).getFloat());
            }


            if(type == VariantType::Pose)
            {
                // TODO:  This comparison is bullshit!
                PosePtr & typedValue =  value.getClass<Pose>();
                PosePtr & param =  getParameter(0).getClass<Pose>();
                return (sqrt(((typedValue->position->x-param->position->x)*(typedValue->position->x-param->position->x))+
                             ((typedValue->position->y-param->position->y)*(typedValue->position->y-param->position->y))+
                             ((typedValue->position->z-param->position->x)*(typedValue->position->x-param->position->z)))+
                            sqrt(((typedValue->orientation->qw-param->orientation->qw)*(typedValue->orientation->qw-param->orientation->qw))+
                             ((typedValue->orientation->qx-param->orientation->qx)*(typedValue->orientation->qx-param->orientation->qx))+
                             ((typedValue->orientation->qy-param->orientation->qy)*(typedValue->orientation->qy-param->orientation->qy))+
                             ((typedValue->orientation->qz-param->orientation->qx)*(typedValue->orientation->qx-param->orientation->qz))) < getParameter(1).getFloat());
            }

           return false;
        }
    };
}

#endif // CONDITIONCHECKEQUALSPOSITIONWITHTOLERANCE_H
