/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    Core
 * @author     Matthias Hadlich (matthias dot hadlich at student dot kit dot edu)
 * @copyright  2014 Matthias Hadlich
 * @license    http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _HARDWAREUNIT_H
#define _HARDWAREUNIT_H

#include <string>

#include <Core/core/Component.h>
#include <Core/core/application/properties/Properties.h>
#include <Core/core/system/ImportExportComponent.h>

#include <RobotAPI/interface/hardware/HardwareInterface.h>

namespace armarx {
  
   class HardwareUnitPropertyDefinitions:
           public ComponentPropertyDefinitions
   {
   public:
       HardwareUnitPropertyDefinitions(std::string prefix):
           ComponentPropertyDefinitions(prefix)
       {
	 
           //defineRequiredProperty<std::string>("RobotNodeSetName","Robot node name");
           //defineRequiredProperty<std::string>("RobotFileName", "Robot file name, e.g. robot_model.xml");
       }
   };
  
class HardwareUnit :
        virtual public HardwareInterface, virtual public Component
    {
    public:
        // inherited from Component
        virtual std::string getDefaultName() const { return "HardwareUnit"; }
        virtual void onInitComponent();
        virtual void onConnectComponent();
        virtual void onExitComponent();
	
        virtual void onInitHardwareUnit() = 0;
        virtual void onConnectHardwareUnit() = 0;
        virtual void onExitHardwareUnit() = 0;
  
         virtual BusNames getConfiguredBusses( const Ice::Current& c = ::Ice::Current());
         virtual BusInformation getBusInformation(const std::string &bus, const Ice::Current& c = ::Ice::Current());
         virtual DeviceNames getDevicesOnBus(const std::string &bus, const Ice::Current& c = ::Ice::Current());
         virtual DeviceInformation getDeviceInformation(const std::string &device,  const Ice::Current& c = ::Ice::Current());

         virtual int performBasicOperation(BasicOperation &operation,  const Ice::Current& c = ::Ice::Current());
         virtual int sendCommand(const std::string &command,  const Ice::Current& c = ::Ice::Current());


        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        virtual PropertyDefinitionsPtr createPropertyDefinitions();
	
	  protected:
	    std::string channel;
	    HardwareInterfacePrx hardwareValuePrx; // gets commands from hardware
	
    };
}




#endif
